const logger = require("./common").createLogger();
const stringify = require("json-stringify");
const playerDisconnection = function(game, playerId) {
  logger.trace(" player %s  disconnected", playerId);
  delete this.players[playerId];
};

const playerJoined = function(game, playerId) {
  logger.trace("player %s joined ", playerId);
  this.players[playerId] = true;
};

const gameObjectsDestroyed = function(game, gameObjectIds, playerId) {
  if (logger.trace()) {
    logger.trace(
      "player %s destroyed fruits %s",
      playerId,
      stringify(gameObjectIds)
    );
  }
  let meta = this.meta;
  let scoreMap = meta.extras.points;
  let totalScore = 0;
  let fruits = game.gameObjectByIds(gameObjectIds);
  for (let [id, fruit] of Object.entries(fruits)) {
    if (!fruit) continue;
    let score = scoreMap[fruit.type];
    if (!score) continue;
    totalScore += score;
  }
  logger.debug("Total score for player %s is %s", playerId, totalScore);
  //
  if (totalScore) {
    game.awardScore(playerId, totalScore, err => {
      if (err) {
        logger.error({ err: err });
        logger.error("neutrino core error , in awarding score");
      }
    });
    logger.debug("score %s is awarded to player %s ", totalScore, playerId);
  }
};

module.exports = { playerDisconnection, playerJoined, gameObjectsDestroyed };
